local rsclass = lib.get_class("resound", "0")

local bstr = function(id, val)
	return val
end

local public = {}

local babel, ref_id, update_class
local do_babel = function()
	if lib.is_ready('babel', "0") then
		lib.log_error("Applying language table to rsplayer")
		babel = lib.get_class("babel")
		ref_id = babel.register("plugins/ResoundPlayer/lang/", {'en', 'es', 'fr', 'pt'})
		
		bstr = function(id, val)
			return babel.fetch(ref_id, id, val)
		end
		
		public.add_translation = function(path, lang_code)
			babel.add_new_lang(ref_id, path, lang_code)
		end
		
		update_class()
	end
end

local config = {
	show_player_in_pda = gkini.ReadString("rsplayer", "show_player_in_pda", "NO"),
}



update_class = function()
	local class = {
		CCD1 = true,
		open = nil,
		config = nil,
		smart_config = {
			title = bstr(1, "Resound Mini Player"),
			cb = function(id, val)
				if config[id] then
					config[id] = val
					gkini.WriteString("rsplayer", id, val)
				end
			end,
			[1] = "notice",
			notice = {
				type = "text",
				display = bstr(2, "rsplayer has limited configuration and functionality. For a fully featured interface to Resound, please check your plugin repository on VOUPR or NexusMods"),
			},
			[2] = "spacer",
			spacer = {
				type = "spacer",
			},
			[3] = "show_player_in_pda",
			show_player_in_pda = {
				type = "toggle",
				display = bstr(3, "Show UI when you dock"),
				[1] = config.show_player_in_pda,
			},
		},
		commands = {
			"/rsplayer",
		},
		manifest = {
			'plugins/ResoundPlayer/rsplayer.lua',
			'plugins/ResoundPlayer/rsplayer.ini',
			'plugins/ResoundPlayer/main.lua',
			'plugins/ResoundPlayer/lang/en.ini',
			'plugins/ResoundPlayer/lang/es.ini',
			'plugins/ResoundPlayer/lang/fr.ini',
			'plugins/ResoundPlayer/lang/pt.ini',
		},
		description = bstr(4, "Resound Mini Player is a minimal interface for the Resound library"),
	}
	
	for k, v in pairs(class) do
		public[k] = v
	end
	
	lib.set_class("rsplayer", "0", public)
end













public.config = function()
	local rsconfig = rsclass.get_settings()
	
	local diag = iup.dialog {
		topmost = "YES",
		fullscreen = "YES",
		bgcolor = "0 0 0 100 *",
		iup.vbox {
			iup.fill { },
			iup.hbox {
				iup.fill { },
				iup.stationbuttonframe {
					iup.vbox {
						alignment = "ACENTER",
						iup.fill {
							size = "%3",
						},
						iup.label {
							title = bstr(16, "Resound Engine Settings"),
							font = Font.H4,
						},
						iup.fill {
							size = "%4",
						},
						iup.hbox {
							iup.label {
								title = bstr(17, "Game Volume: "),
							},
							iup.canvas {
								size = "200x" .. tostring(Font.Default),
								border = "NO",
								xmin = 0,
								xmax = 100,
								dx = 10,
								posx = rsconfig.vol * 100,
								expand = "NO",
								scrollbar = "HORIZONTAL",
								scroll_cb = function(self)
									local apply_timer = Timer()
									apply_timer:SetTimeout(1, function()
										rsclass.set_setting("vol", self.posx / 100)
										Game.SetCVar("snd_musicvolume", tostring(self.posx / 100))
									end)
								end,
							},
						},
						iup.fill {
							size = "%2",
						},
						iup.hbox {
							iup.label {
								title = bstr(18, "Scanning Tick Interval (ms): "),
							},
							iup.text {
								size = "200x",
								value = rsconfig.tick_time,
								action = function(self)
									local val = tonumber(self.value) or 0
									val = val > 0 and val or 0
									self.value = tostring(val)
									gkini.WriteString("Resound", "tick_time", self.value)
									rsclass.set_setting("tick_time", self.value)
								end,
							},
						},
						iup.fill {
							size = "%2",
						},
						iup.hbox {
							iup.label {
								title = bstr(23, "Seek method") .. ":  ",
							},
							iup.stationsublist {
								[1] = bstr(24, "Iterative: won't hang during long seeks"),
								[2] = bstr(25, "Hard Loop: more efficient"),
								dropdown = "YES",
								value = (rsconfig.seek_style == "iter" and 1) or 2,
								action = function(self, t, i, c)
									if c == 1 then
										local new_val = i == 1 and "iter" or "hard"
										gkini.WriteString("Resound", "seek_style", new_val)
										rsclass.set_setting("seek_style", new_val)
									end
									return iup.DEFAULT
								end,
							},
						},
						iup.fill {
							size = "%2",
						},
						iup.hbox {
							iup.label {
								title = bstr(19, "Seek attempts max") .. ":  ",
							},
							iup.text {
								size = "200x",
								value = rsconfig.seek_songs,
								action = function(self)
									local val = tonumber(self.value) or 0
									val = val > 0 and val or 0
									self.value = tostring(val)
									gkini.WriteString("Resound", "seek_songs", self.value)
									rsclass.set_setting("seek_songs", self.value)
								end,
							},
						},
						iup.fill {
							size = "%2",
						},
						iup.hbox {
							iup.label {
								title = bstr(20, "Use resound bonus tags") .. ":  ",
							},
							iup.stationtoggle {
								title = "",
								value = rsconfig.adv_tags == "YES" and "ON" or "OFF",
								action = function(self)
									gkini.WriteString("Resound", "adv_tags", self.value == "ON" and "YES" or "NO")
									rsclass.set_setting("adv_tags", self.value == "ON" and "YES" or "NO")
								end,
							},
						},
						iup.fill {
							size = "%2",
						},
						iup.hbox {
							iup.label {
								title = bstr(21, "Use LME notifications on track change" .. ":  "),
							},
							iup.stationtoggle {
								title = "",
								value = rsconfig.do_notifs == "YES" and "ON" or "OFF",
								action = function(self)
									gkini.WriteString("Resound", "do_notifs", self.value == "ON" and "YES" or "NO")
									rsclass.set_setting("do_notifs", self.value == "ON" and "YES" or "NO")
								end,
							},
						},
						iup.fill {
							size = "%2",
						},
						iup.hbox {
							iup.label {
								title = bstr(22, "Show chat messages on track change" .. ":  "),
							},
							iup.stationtoggle {
								title = "",
								value = rsconfig.do_chat == "YES" and "ON" or "OFF",
								action = function(self)
									gkini.WriteString("Resound", "do_chat", self.value == "ON" and "YES" or "NO")
									rsclass.set_setting("do_chat", self.value == "ON" and "YES" or "NO")
								end,
							},
						},
						iup.fill {
							size = "%4",
						},
						iup.hbox {
							iup.fill { },
							iup.button {
								title = bstr(8, "Close"),
								action = function(self)
									HideDialog(iup.GetDialog(self))
								end,
							},
						},
					}
				},
				iup.fill { },
			},
			iup.fill { },
		},
	}
	
	ShowDialog(diag)
end













local update_player = function() end

public.open = function()
	
	local song_name = iup.label {
		title = bstr(9, "Unknown song"),
		font = Font.H6,
	}

	local song_artist = iup.label {
		title = bstr(10, "Unknown artist"),
		font = Font.H6,
	}

	local song_album = iup.label {
		title = bstr(11, "Unknown album"),
		font = Font.H6,
	}
	
	
	
	
	
	local song_select = {}

	local song_list = {}
	local song_view = iup.stationsublist {
		size = "%50x%15",
		action = function(self, t, i, v)
			song_select[2] = t or "untitled"
		end,
	}

	local album_toggle = iup.stationtoggle {
		title = "<album>",
		value = "OFF",
		action = function(self)
			if self.value == "ON" then
				rsclass.toggle_album_playback(song_select[1], "YES")
			else
				rsclass.toggle_album_playback(song_select[1], "NO")
			end
		end
	}

	local album_list = {}
	local album_view = iup.stationsublist {
		dropdown = "YES",
		size = "%50x",
		action = function(self, t, i, v)
			for i, _ in ipairs(song_list) do
				song_view[i] = nil
			end
			song_list = rsclass.get_songs(t)
			for i, v in ipairs(song_list) do
				song_view[i] = tostring(v)
			end
			iup.Refresh(song_view)
			song_select[1] = t or "ERROR"
			album_toggle.title = t or "ERROR"
			album_toggle.value = rsclass.toggle_album_playback(t, nil) == "YES" and "ON" or "OFF"
			iup.Refresh(album_toggle)
		end
	}
	
	local album_diag = iup.dialog {
		topmost = "YES",
		fullscreen = "YES",
		bgcolor = "0 0 0 100 *",
		iup.vbox {
			iup.fill { },
			iup.hbox {
				iup.fill { },
				iup.stationbuttonframe {
					iup.vbox {
						alignment = "ACENTER",
						iup.fill {
							size = "%2",
						},
						iup.label {
							title = bstr(5, "Resound Mini Player: Album List and Options"),
							font = Font.Tab
						},
						iup.fill {
							size = "%3",
						},
						iup.hbox {
							iup.label {
								title = bstr(12, "Albums in Resound:"),
							},
							iup.fill {
								size = "%30",
							},
						},
						album_view,
						iup.hbox {
							--iup.fill { },
							iup.label {
								title = bstr(6, "Album will be loaded for playback: "),
							},
							album_toggle,
							
						},
						iup.fill {
							size = "%1",
						},
						iup.hbox {
							iup.label {
								title = bstr(13, "Songs in album:"),
							},
							iup.fill {
								size = "%30",
							},
						},
						song_view,
						iup.hbox {
							iup.fill {
								size = "%30",
							},
							iup.button {
								title = bstr(7, "Play selected track"),
								action = function()
									if not song_select[1] then
										song_select[1] = "ERROR"
									end
									if not song_select[2] then
										song_select[2] = "ERROR"
									end
									rsclass.play(song_select)
								end,
							},
						},
						iup.fill {
							size = "%4",
						},
						iup.stationsubframe {
							iup.hbox {
								iup.button {
									--seek/pause control
									title = "||", --replace with image
									current_val = ">",
									size = "48x48",
									action = function(self)
										if self.current_val == ">" then
											rsclass.stop()
											self.title = ">" --replace with play image
											self.current_val = "="
										else
											rsclass.play({song_album.title, song_name.title})
											self.title = "||" --replace with pause image
											self.current_val = ">"
										end
									end,
								},
								iup.button {
									--seek control
									title = "?",
									size = "48x48",
									action = function(self)
										rsclass.play()
									end,
								},
								iup.label {
									title = " ",
								},
								iup.vbox {
									song_name,
									song_artist,
									song_album,
								},
							},
						},
						iup.fill {
							size = "%1",
						},
						
						iup.hbox {
							iup.button {
								title = bstr(15, "More Options"),
								action = function()
									public.config()
								end,
							},
							iup.fill {
								size = "%3",
							},
							iup.stationtoggle {
								title = bstr(3, ": Show player when docked"),
								value = config.show_player_in_pda == "YES" and "ON" or "OFF",
								action = function(self)
									if self.value == "ON" then
										config.show_player_in_pda = "YES"
									else
										config.show_player_in_pda = "NO"
									end
									gkini.WriteString("rsplayer", "show_player_in_pda", config.show_player_in_pda)
								end,
							},
							iup.fill {
								size = "%3",
							},
							iup.stationtoggle {
								title = bstr(14, "Enable Resound"),
								value = (rsclass.get_settings().enable_resound == "YES") and "ON" or "OFF",
								action = function(self)
									if self.value == "ON" then
										rsclass.toggle_resound("YES")
									else
										rsclass.toggle_resound("NO")
									end
								end,
							},
							iup.fill {
								size = "%3",
							},
							iup.button {
								title = bstr(8, "Close"),
								action = function(self)
									HideDialog(iup.GetDialog(self))
								end,
							},
						},
					},
				},
				iup.fill { },
			},
			iup.fill { },
		},
	}
	
	album_diag:map()
	
	
	
	update_player = function()
		local current_track = rsclass.get_playing()
		song_name.title = current_track.name
		song_album.title = current_track.album
		song_artist.title = current_track.artist
		
		iup.Refresh(album_diag)
	end
	
	update_player()
	
	
	for k, v in ipairs(album_list) do
		album_view[k] = nil
	end
	album_list = rsclass.get_albums()
	for k, v in ipairs(album_list) do
		album_view[k] = tostring(v)
	end
	
	album_view:action(album_list[1] or "ERROR", 1, -100)
	ShowDialog(album_diag)
end


rsclass.on_update(function() 
	if type(update_player) == "function" then
		update_player()
	end
end)


RegisterUserCommand("rsplayer", function()
	public.open()
end)

RegisterEvent(function()
	if config.show_player_in_pda == "YES" then
		public.open()
	end
end, "ENTERED_STATION")

update_class()

lib.require({{name="babel", version="0"}}, do_babel)